﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace SmdICpad
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
            icpadPram = new SmdICpadParam();
            mbeClipboardFormat = DataFormats.GetFormat("MbeClipboard-v1.0");
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            colorTextBack = textBoxXPin.BackColor;
            SetFormVal();
        }

        private void SetFormVal()
        {
            textBoxXPin.Text = Convert.ToString(icpadPram.xPinCount);
            textBoxYPin.Text = Convert.ToString(icpadPram.yPinCount);
            textBoxX1.Text = String.Format("{0:##0.0###}", icpadPram.x1);
            textBoxX2.Text = String.Format("{0:##0.0###}", icpadPram.x2);
            textBoxY1.Text = String.Format("{0:##0.0###}", icpadPram.y1);
            textBoxY2.Text = String.Format("{0:##0.0###}", icpadPram.y2);
            textBoxW.Text = String.Format("{0:##0.0###}", icpadPram.padWidth);
            textBoxP.Text = String.Format("{0:##0.0###}", icpadPram.pinPitch);
            radioButtonQFP.Checked = (icpadPram.pkgtype == SmdICpadParam.packageType.QFP);
        }

        private SmdICpadParam icpadPram;
        private Color colorTextBack;

        public static  DataFormats.Format mbeClipboardFormat;



        private bool checkParam()
        {
            Color errColor = Color.Orange;
            bool retval = true;
            bool x1err = false;
            bool y1err = false;

            int n;
            double d;

            //X ピン数
            try {
                n = Convert.ToInt32(textBoxXPin.Text);
            }
            catch {
                n = -1;
            }
            if (n < SmdICpadCore.MIN_X_PINCOUNT || n > SmdICpadCore.MAX_X_PINCOUNT) {
                textBoxXPin.BackColor = errColor;
                retval = false;
            } else {
                icpadPram.xPinCount = n;
            }

            //パッケージタイプ
            icpadPram.pkgtype = (radioButtonQFP.Checked ? SmdICpadParam.packageType.QFP : SmdICpadParam.packageType.PLCC);

            //Y ピン数
            try {
                n = Convert.ToInt32(textBoxYPin.Text);
            }
            catch {
                n = -1;
            }
            if (n < SmdICpadCore.MIN_Y_PINCOUNT || n > SmdICpadCore.MAX_Y_PINCOUNT || 
                (icpadPram.pkgtype==SmdICpadParam.packageType.PLCC && ((n&1)==0))){
                textBoxYPin.BackColor = errColor;
                retval = false;
            } else {
                icpadPram.yPinCount = n;
            }

            //X1 寸法
            try {
                d = Convert.ToDouble(textBoxX1.Text);
            }
            catch {
                d = 0;
            }
            if(d<SmdICpadCore.MIN_XY1 || d > SmdICpadCore.MAX_XY1) {
                textBoxX1.BackColor = errColor;
                x1err = true;
                retval = false;
            } else {
                icpadPram.x1 = d;
            }

            //X2 寸法
            try {
                d = Convert.ToDouble(textBoxX2.Text);
            }
            catch {
                d = 0.0;
            }
            if (x1err || d < SmdICpadCore.min_XY2(icpadPram.x1) || d > SmdICpadCore.max_XY2(icpadPram.x1)) {
                textBoxX2.BackColor = errColor;
                retval = false;
            } else {
                textBoxX2.BackColor = colorTextBack;
                icpadPram.x2 = d;
            }

            //Y1 寸法
            try {
                d = Convert.ToDouble(textBoxY1.Text);
            }
            catch {
                d = 0.0;
            }
            if (d < SmdICpadCore.MIN_XY1 || d > SmdICpadCore.MAX_XY1) {
                textBoxY1.BackColor = errColor;
                y1err = true;
                retval = false;
            } else {
                icpadPram.y1 = d;
            }

            //Y2 寸法
            try {
                d = Convert.ToDouble(textBoxY2.Text);
            }
            catch {
                d = 0.0;
            }
            if (y1err || d < SmdICpadCore.min_XY2(icpadPram.y1) || d > SmdICpadCore.max_XY2(icpadPram.y1)) {
                textBoxY2.BackColor = errColor;
                retval = false;
            } else {
                textBoxY2.BackColor = colorTextBack;
                icpadPram.y2 = d;
            }

            //W 寸法
            try {
                d = Convert.ToDouble(textBoxW.Text);
            }
            catch {
                d = 0.0;
            }
            if (d < SmdICpadCore.MIN_PAD_WIDTH || d > SmdICpadCore.MAX_PAD_WIDTH) {
                textBoxW.BackColor = errColor;
                retval = false;
            } else {
                icpadPram.padWidth = d;
            }

            //P 寸法
            try {
                d = Convert.ToDouble(textBoxP.Text);
            }
            catch {
                d = 0.0;
            }
            if (d < SmdICpadCore.MIN_PIN_PITCH || d > SmdICpadCore.MAX_PIN_PITCH) {
                textBoxP.BackColor = errColor;
                retval = false;
            } else {
                icpadPram.pinPitch = d;
            }

            return retval;
        }


        private void buttonCopy_Click(object sender, EventArgs e)
        {
            if (!checkParam()) {
                MessageBox.Show("Invalid parameter(s)");
                return;
            }
            SmdICpadCore.DoCopy(icpadPram);

        }

        private void textBoxXPin_TextChanged(object sender, EventArgs e)
        {
            textBoxXPin.BackColor = colorTextBack;
        }

        private void textBoxYPin_TextChanged(object sender, EventArgs e)
        {
            textBoxYPin.BackColor = colorTextBack;
        }

        private void textBoxX1_TextChanged(object sender, EventArgs e)
        {
            textBoxX1.BackColor = colorTextBack;
        }

        private void textBoxX2_TextChanged(object sender, EventArgs e)
        {
            textBoxX2.BackColor = colorTextBack;
        }

        private void textBoxY1_TextChanged(object sender, EventArgs e)
        {
            textBoxY1.BackColor = colorTextBack;
        }

        private void textBoxY2_TextChanged(object sender, EventArgs e)
        {
            textBoxY2.BackColor = colorTextBack;
        }

        private void textBoxP_TextChanged(object sender, EventArgs e)
        {
            textBoxP.BackColor = colorTextBack;
        }

        private void textBoxW_TextChanged(object sender, EventArgs e)
        {
            textBoxW.BackColor = colorTextBack;
        }

        private void buttonExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void buttonAbout_Click(object sender, EventArgs e)
        {
            AboutBox dlg = new AboutBox();
            dlg.ShowDialog();
        }
 
    }
}
